@@ -499,3 +499,306 @@ xyz_to_hex <- function(xyz, transformation = "sRGB", linear_func = NULL) {
     hex <- rgb_to_hex(rgb)
     return(hex)
 }
+
+### RGB to HSL
+#' Convert RGB space into HSL space
+#' @export
+#' @param rgb A dataframe or matrix with red, green and blue colour channels located in the columns 1 to 3, respectively. Colour channel values should be between 0 and 255, inclusive.
+#' @return a \code{tibble} of H, S and L colour channels.
+#' @examples
+#' red <- sample(x = 1:255, size = 10, replace = TRUE)
+#' green <- sample(x = 1:255, size = 10, replace = TRUE)
+#' blue <- sample(x = 1:255, size = 10, replace = TRUE)
+#' rgb_to_hsl(data.frame(r = red, g = green, b = blue))
+rgb_to_hsl <- function(rgb) {
+
+    # Convert rbg into [0, 1]
+    temp_rgb <- rgb / 255
+
+    # Computing constants
+    M <- apply(X = temp_rgb, MARGIN = 1, FUN = max)
+    m <- apply(X = temp_rgb, MARGIN = 1, FUN = min)
+    C <- M - m
+
+    ### Calculating H
+    # Finding H'
+    H_dash <-
+        case_when(
+            C == 0 ~ NA_real_,
+            M == temp_rgb[ , 1] ~ ((temp_rgb[ , 2] - temp_rgb[ , 3]) / C) %% 6,
+            M == temp_rgb[ , 2] ~ ((temp_rgb[ , 3] - temp_rgb[ , 1]) / C) + 2,
+            TRUE ~ ((temp_rgb[ , 1] - temp_rgb[ , 2]) / C) + 4
+        )
+
+    # Calculate H = 60deg * H'
+    H <- H_dash * 60
+
+    ### Calculating L
+    L <- (M + m) / 2
+
+    ### Calculating S
+    S <- 
+        case_when(
+            (L == 0) | (L == 1) ~ 0,
+            TRUE ~ C / (1 - abs(2 * L - 1))
+        )
+    
+    # Package together in one dataframe
+    hsl <- tibble(
+        h = H,
+        s = S,
+        l = L
+    )
+
+    return(hsl)
+
+}
+
+### HSL to RGB 
+#' Convert HSL space into RGB space
+#' @export
+#' @param hsl A dataframe or matrix with H, S and L colour channels located in the columns 1 to 3, respectively. H in degrees in [0, 360], S and L in [0, 1]
+#' @return A \code{tibble} of red, green and blue colour channels.
+#' @examples
+#' H <- sample(x = 0:360, size = 10, replace = TRUE)
+#' S <- runif(n = 10)
+#' L <- runif(n = 10)
+#' hsl_to_rgb(data.frame(h = H, s = S, l = L))
+hsl_to_rgb <- function(hsl) {
+    # Unlist from hsl
+    temp_h <- unlist(hsl[ , 1])
+    temp_s <- unlist(hsl[ , 2])
+    temp_l <- unlist(hsl[ , 3])
+
+    # Calculate parameters
+    C <- (1 - abs(2 * temp_l - 1)) * temp_s
+    H_dash <- temp_h / 60
+    X <- C * (1 - abs((H_dash %% 2) - 1))
+    m <- temp_l - (C / 2)
+
+    # Create a temporary dataframe
+    cxmhash <- tibble(
+        C = C,
+        X = X,
+        m = m,
+        H_dash = H_dash
+    )
+
+    # Calculate RGB
+    rgb <- cxmhash 
+    rgb$r <- case_when(
+                ((rgb$H_dash >= 0) & (rgb$H_dash < 1)) | ((rgb$H_dash >= 5) & (rgb$H_dash < 6)) ~ C + m,
+                ((rgb$H_dash >= 1) & (rgb$H_dash < 2)) | ((rgb$H_dash >= 4) & (rgb$H_dash < 5))  ~ rgb$X + rgb$m,
+                TRUE ~ rgb$m          
+                )
+    rgb$g <- case_when(
+                (rgb$H_dash >= 1) & (rgb$H_dash < 3) ~ rgb$C + rgb$m,
+                ((rgb$H_dash >= 0) & (rgb$H_dash < 1)) | ((rgb$H_dash >= 3) & (rgb$H_dash < 4)) ~ rgb$X + rgb$m,
+                TRUE ~ rgb$m          
+            )
+    rgb$b = case_when(
+                (rgb$H_dash >= 0) & (rgb$H_dash < 2) ~ rgb$m,
+                (rgb$H_dash >= 3) & (rgb$H_dash < 5) ~ rgb$C + rgb$m,
+                TRUE ~ X + m          
+            )
+        
+    rgb <- rgb[, c("r", "g", "b")] |>
+        mutate(
+            across(.cols = c("r", "g", "b"), function(x) if_else((x < 0) & (abs(x) < 1e-14), 0, x)) # Correcting for machine precision negative values
+        ) * 255
+    
+    return(rgb)
+}
+
+### HSL to HSV
+#' Convert HSL to HSV
+#' @export
+#' @param hsl A dataframe or matrix with H, S and L colour channels located in the columns 1 to 3, respectively. H in degrees in [0, 360], S and L in [0, 1]
+#' @return A \code{tibble} of H, S and V colour channels. Hue is constant between colour spaces, while saturation differs.
+#' @examples
+#' H <- sample(x = 0:360, size = 10, replace = TRUE)
+#' S <- runif(n = 10)
+#' L <- runif(n = 10)
+#' hsl_to_hsv(data.frame(h = H, s = S, l = L))
+hsl_to_hsv <- function(hsl) {
+    temp_h <- unlist(hsl[ , 1])
+    temp_s <- unlist(hsl[ , 2])
+    temp_l <- unlist(hsl[ , 3])
+
+    # Calculate v
+    V <- temp_l + temp_s * pmin(temp_l, 1 - temp_l)
+
+    # Calculate S
+    S = case_when(
+        V == 0 ~ 0,
+        TRUE ~ 2 * (1 - (temp_l / V))
+    )
+
+    hsv <- tibble(
+        h = temp_h,
+        s = S,
+        v = V
+    )
+
+    return(hsv)
+}
+
+### HSV to HSL
+#' Convert HSV to HSL
+#' @export
+#' @param hsv A dataframe or matrix with H, S and V colour channels located in the columns 1 to 3, respectively. H in degrees in [0, 360], S and L in [0, 1]
+#' @return A \code{tibble} of H, S and L colour channels. Hue is constant between colour spaces, while saturation differs.
+#' @examples
+#' H <- sample(x = 0:360, size = 10, replace = TRUE)
+#' S <- runif(n = 10)
+#' V <- runif(n = 10)
+#' hsv_to_hsl(data.frame(h = H, s = S, v = V))
+hsv_to_hsl <- function(hsv) {
+    temp_h <- unlist(hsv[ , 1])
+    temp_s <- unlist(hsv[ , 2])
+    temp_v <- unlist(hsv[ , 3])
+
+    # Calculate L
+    L <- temp_v * (1 - (temp_s / 2))
+
+    # Calculate S
+    S <- case_when(
+        (L == 0) | (L == 1) ~ 0,
+        TRUE ~ (temp_v - L) / (pmin(L, 1 - L))
+    )
+
+    hsl <- tibble(
+        h = temp_h,
+        s = S,
+        l = L
+    )
+
+    return(hsl)
+
+}
+
+### RGB to HSV
+#' Convert RGB to HSV
+#' @export
+#' @param rgb A dataframe or matrix with red, green and blue colour channels located in the columns 1 to 3, respectively. Colour channel values should be between 0 and 255, inclusive.
+#' @return A \code{tibble} of H, S and V colour channels.
+rgb_to_hsv <- function(rgb) {
+    hsl <- rgb_to_hsl(rgb)
+    hsv <- hsl_to_hsv(hsl)
+    return(hsv)
+}
+
+### HSV to RGB
+#' Convert HSV to RGB
+#' @export
+#' @param hsv A dataframe or matrix with H, S and V colour channels located in the columns 1 to 3, respectively. H in degrees in [0, 360], S and L in [0, 1]
+#' @return A \code{tibble} of red, green and blue colour channels.
+hsv_to_rgb <- function(hsv) {
+    hsl <- hsv_to_hsl(hsv)
+    rgb <- hsl_to_rgb(hsl)
+    return(rgb)
+}
+
+### XYZ to HSL
+#' Convert XYZ to HSL
+#' @export
+#' @param xyz A dataframe or matrix with X, Y and Z colour channels located in the columns 1 to 3, respectively.
+#' @param transformation An option in \code{c("sRGB", "Adobe")} for a built-in transformation or, alternatively, a custom 3x3 transformation matrix.
+#' @param linear_func A function to convert RGB colour space into linear RGB space. Used only if a custom transformation matrix is provided. Transformation skips if no function is provided under a user-defined transformation matrix. See: https://en.wikipedia.org/wiki/SRGB.
+#' @return A \code{tibble} of H, S and L colour channels.
+xyz_to_hsl <- function(xyz, transformation = "sRGB", linear_func = NULL) {
+    rgb <- xyz_to_rgb(xyz, transformation = transformation, linear_func = linear_func)
+    hsl <- rgb_to_hsl(rgb)
+    return(hsl)
+}
+
+### HSL to XYZ
+#' Convert HSL to XYZ
+#' @export
+#' @param hsl A dataframe or matrix with H, S and L colour channels located in the columns 1 to 3, respectively. H in degrees in [0, 360], S and L in [0, 1]
+#' @param transformation An option in \code{c("sRGB", "Adobe")} for a built-in transformation or, alternatively, a custom 3x3 transformation matrix.
+#' @param linear_func A function to convert RGB colour space into linear RGB space. Used only if a custom transformation matrix is provided. Transformation skips if no function is provided under a user-defined transformation matrix. See: https://en.wikipedia.org/wiki/SRGB.
+#' @return A \code{tibble} of X, Y and Z colour channels.
+hsl_to_xyz <- function(hsl, transformation = "sRGB", linear_func = NULL) {
+    rgb <- hsl_to_rgb(hsl)
+    xyz <- rgb_to_xyz(rgb, transformation = transformation, linear_func = linear_func)
+    return(xyz)
+}
+
+### XYZ to HSV
+#' Convert XYZ to HSV
+#' @export
+#' @param xyz A dataframe or matrix with X, Y and Z colour channels located in the columns 1 to 3, respectively.
+#' @param transformation An option in \code{c("sRGB", "Adobe")} for a built-in transformation or, alternatively, a custom 3x3 transformation matrix.
+#' @param linear_func A function to convert RGB colour space into linear RGB space. Used only if a custom transformation matrix is provided. Transformation skips if no function is provided under a user-defined transformation matrix. See: https://en.wikipedia.org/wiki/SRGB.
+#' @return A \code{tibble} of H, S and V colour channels.
+xyz_to_hsv <- function(xyz, transformation = "sRGB", linear_func = NULL) {
+    rgb <- xyz_to_rgb(xyz, transformation = transformation, linear_func = linear_func)
+    hsv <- rgb_to_hsv(rgb)
+    return(hsv)
+}
+
+### HSV to XYZ
+#' Convert HSV to XYZ
+#' @export
+#' @param hsv A dataframe or matrix with H, S and V colour channels located in the columns 1 to 3, respectively. H in degrees in [0, 360], S and L in [0, 1]
+#' @param transformation An option in \code{c("sRGB", "Adobe")} for a built-in transformation or, alternatively, a custom 3x3 transformation matrix.
+#' @param linear_func A function to convert RGB colour space into linear RGB space. Used only if a custom transformation matrix is provided. Transformation skips if no function is provided under a user-defined transformation matrix. See: https://en.wikipedia.org/wiki/SRGB.
+#' @return A \code{tibble} of X, Y and Z colour channels.
+hsv_to_xyz <- function(hsv, transformation = "sRGB", linear_func = NULL) {
+    rgb <- hsv_to_rgb(hsv)
+    xyz <- rgb_to_xyz(rgb, transformation = transformation, linear_func = linear_func)
+    return(xyz)
+}
+
+### HSL to Lab
+#' Convert HSL to Lab
+#' @export
+#' @param hsl A dataframe or matrix with H, S and L colour channels located in the columns 1 to 3, respectively. H in degrees in [0, 360], S and L in [0, 1]
+#' @param transformation An option in \code{c("sRGB", "Adobe")} for a built-in transformation or, alternatively, a custom 3x3 transformation matrix.
+#' @param linear_func A function to convert RGB colour space into linear RGB space. Used only if a custom transformation matrix is provided. Transformation skips if no function is provided under a user-defined transformation matrix. See: https://en.wikipedia.org/wiki/SRGB.
+#' @return A \code{tibble} of L, a and b colour space values.
+hsl_to_lab <- function(hsl, transformation = "sRGB", linear_func = NULL) {
+    rgb <- hsl_to_rgb(hsl)
+    lab <- rgb_to_lab(rgb, transformation = transformation, linear_func = linear_func)
+    return(lab)
+}
+
+### Lab to HSL
+#' Convert Lab to HSL
+#' @export
+#' @param lab A dataframe or matrix with L, a and b colour channels located in the columns 1 to 3, respectively.
+#' @param transformation An option in \code{c("sRGB", "Adobe")} for a built-in transformation or, alternatively, a custom 3x3 transformation matrix.
+#' @param linear_func A function to convert RGB colour space into linear RGB space. Used only if a custom transformation matrix is provided. Transformation skips if no function is provided under a user-defined transformation matrix. See: https://en.wikipedia.org/wiki/SRGB.
+#' @return A \code{tibble} of H, S and L colour channels.
+lab_to_hsl <- function(lab, transformation = "sRGB", linear_func = NULL) {
+    rgb <- lab_to_rgb(lab, transformation = transformation, linear_func = linear_func)
+    hsl <- rgb_to_hsl(rgb)
+    return(hsl)
+}
+
+### HSV to Lab
+#' Convert HSV to Lab 
+#' @export
+#' @param hsv A dataframe or matrix with H, S and V colour channels located in the columns 1 to 3, respectively. H in degrees in [0, 360], S and L in [0, 1]
+#' @param transformation An option in \code{c("sRGB", "Adobe")} for a built-in transformation or, alternatively, a custom 3x3 transformation matrix.
+#' @param linear_func A function to convert RGB colour space into linear RGB space. Used only if a custom transformation matrix is provided. Transformation skips if no function is provided under a user-defined transformation matrix. See: https://en.wikipedia.org/wiki/SRGB.
+#' @return A \code{tibble} of L, a and b colour space values.
+hsv_to_lab <- function(hsv, transformation = "sRGB", linear_func = NULL) {
+    hsl <- hsv_to_hsl(hsv)
+    lab <- hsl_to_lab(hsl, transformation = transformation, linear_func = linear_func)
+    return(lab)
+}
+
+### Lab to HSV
+#' Convert Lab to HSv
+#' @export
+#' @param lab A dataframe or matrix with L, a and b colour channels located in the columns 1 to 3, respectively.
+#' @param transformation An option in \code{c("sRGB", "Adobe")} for a built-in transformation or, alternatively, a custom 3x3 transformation matrix.
+#' @param linear_func A function to convert RGB colour space into linear RGB space. Used only if a custom transformation matrix is provided. Transformation skips if no function is provided under a user-defined transformation matrix. See: https://en.wikipedia.org/wiki/SRGB.
+#' @return A \code{tibble} of H, S and V colour channels.
+lab_to_hsv <- function(lab, transformation = "sRGB", linear_func = NULL) {
+    hsl <- lab_to_hsl(lab, transformation = transformation, linear_func = linear_func)
+    hsv <- hsl_to_hsv(hsl)
+    return(hsv)
+}
\ No newline at end of file
